// The following functions are intended to be useful for any application:
//
// clone_media_offer(), clone_media_descriptions(), clone_payloads()
// clone_audio_video_payloads(), clone_image_payloads(), clone_control_payloads(),
// clone_misc_attributes(), 
// free_media_offer(), free_media_descriptions(), free_payloads(),
// free_audio_video_payloads(), free_image_payloads() & free_misc_attributes()
//
// These following functions are more intended for illustrative purposes only. 
// There is likely to be rather more logic involved in a real application:
//
// populate_media_answer(), populate_media_offer(), codec_is_supported(),
// convert_payload_number_to_name(), 
//
// The function print_media_offer() is used for display purposes only and is 
// unlikely to be of use in a real application.


#include "SDPhandling.h"

ACU_CHAR local_connection_address[16];
ACU_UINT local_rtp_port;

/*------------------------------------------------------------------------------\
|                                                                               |
|   clone_media_offer()                                                         |
|                                                                               |
|   Copy all elements of a media offer for later use.                           |
|                                                                               |
\------------------------------------------------------------------------------*/
ACU_MEDIA_OFFER_ANSWER *clone_media_offer(const ACU_MEDIA_OFFER_ANSWER* offer)
{
   ACU_MEDIA_OFFER_ANSWER *temp;

   if(NULL == offer)
   {
      // No media offer present
      return NULL;
   }

   temp = (ACU_MEDIA_OFFER_ANSWER*)malloc(sizeof(ACU_MEDIA_OFFER_ANSWER));
   memset(temp, 0, sizeof(ACU_MEDIA_OFFER_ANSWER));

   // Copy connection address information
   temp->connection_address.address_type = offer->connection_address.address_type;
   if(NULL != offer->connection_address.address)
   {
      temp->connection_address.address = (ACU_CHAR*)malloc(strlen(offer->connection_address.address) + 1);
      strcpy(temp->connection_address.address, offer->connection_address.address);
   }
	
   // Copy raw SDP
   if(NULL != offer->raw_sdp)
   {
      temp->raw_sdp = (ACU_CHAR*)malloc(strlen(offer->raw_sdp) + 1);
      strcpy(temp->raw_sdp, offer->raw_sdp);
   }

   temp->media_descriptions = clone_media_descriptions(offer->media_descriptions);

   return temp;
}

/*------------------------------------------------------------------------------\
|                                                                               |
|   clone_media_descriptions()                                                  |
|                                                                               |
|   Copy all media descriptions for a given media offer.                        |
|                                                                               |
\------------------------------------------------------------------------------*/
ACU_MEDIA_DESCRIPTION *clone_media_descriptions(ACU_MEDIA_DESCRIPTION *media_descriptions)
{
   ACU_MEDIA_DESCRIPTION *new_desc;

   if(NULL == media_descriptions)
   {
      // There are no media descriptions to copy
      return NULL;
   }

   // Although there may be several media descriptions this function is
   // recursive so only enough memory for one media description needs to 
   // be allocated here.
   new_desc = (ACU_MEDIA_DESCRIPTION*)malloc(sizeof(ACU_MEDIA_DESCRIPTION));
   memset(new_desc, 0, sizeof(ACU_MEDIA_DESCRIPTION));

   if(NULL != media_descriptions->next)
   {
      // The next media description in the list is not NULL
      // so it needs to be copied first.
      new_desc->next = clone_media_descriptions(media_descriptions->next);
   }

   // Now copy all of the media attributes contained in the media description
   new_desc->connection_address.address_type = media_descriptions->connection_address.address_type;
   if(NULL != media_descriptions->connection_address.address)
   {
      new_desc->connection_address.address = (ACU_CHAR*)malloc(MAXADDR);
      strcpy(new_desc->connection_address.address, media_descriptions->connection_address.address);
   }
   if(NULL != media_descriptions->transport)
   {
      new_desc->transport = (ACU_CHAR*)malloc(strlen(media_descriptions->transport) + 1);
      strcpy(new_desc->transport, media_descriptions->transport);
   }
   new_desc->packet_length    = media_descriptions->packet_length;
   new_desc->media_direction  = media_descriptions->media_direction;
   new_desc->media_type       = media_descriptions->media_type;
   new_desc->port             = media_descriptions->port;

   // Copy the payloads given in this media description
   new_desc->payloads = clone_payloads(media_descriptions->payloads, media_descriptions->media_type);

   // Copy any miscellaneous attributes
   new_desc->miscellaneous_attributes = clone_misc_attributes(media_descriptions->miscellaneous_attributes);

   return new_desc;
}
/*------------------------------------------------------------------------------\
|                                                                               |
|   clone_payloads()                                                            |
|                                                                               |
|   Check media type and copy all payloads in the list.                         |
|                                                                               |
\------------------------------------------------------------------------------*/
ACU_PAYLOAD *clone_payloads(const ACU_PAYLOAD *payloads, const ACU_MEDIA_TYPES media_type)
{
   ACU_PAYLOAD *new_payload = NULL;

   if(NULL == payloads)
   {
      // There are no payloads to copy
      return NULL;
   }

   switch(media_type)
   {
   case ACU_AUDIO:
   case ACU_VIDEO:
      new_payload = clone_audio_video_payloads(payloads);
      break;
   case ACU_IMAGE:
      new_payload = clone_image_payloads(payloads);
      break;
   case ACU_CONTROL:
      new_payload = clone_control_payloads(payloads);
      break;
   case ACU_TEXT:
   case ACU_APPLICATION:
   default:
      // Nothing to copy
      break;
   }

   return new_payload;
}

/*------------------------------------------------------------------------------\
|                                                                               |
|   clone_audio_video_payloads()                                                 |
|                                                                               |
|   Copy payloads for media types ACU_AUDIO and ACU_VIDEO                        |
|                                                                               |
\------------------------------------------------------------------------------*/
ACU_PAYLOAD *clone_audio_video_payloads(const ACU_PAYLOAD *payloads)
{
   ACU_PAYLOAD *new_payload = NULL;
   
   // Although there may be several payloads this function is recursive
   // so only enough memory for one payload needs to be allocated here.
   new_payload = (ACU_PAYLOAD*)malloc(sizeof(ACU_PAYLOAD));
   memset(new_payload, 0, sizeof(ACU_PAYLOAD));

   if(NULL != payloads->next)
   {
      // The next payload in the list is not NULL so it needs to be copied first.
      new_payload->next = clone_audio_video_payloads(payloads->next);
   }

   // Copy payload attributes
   new_payload->payload.audio_video.clock_rate     = payloads->payload.audio_video.clock_rate;
   new_payload->payload.audio_video.packet_length  = payloads->payload.audio_video. packet_length;
   if(NULL != payloads->payload.audio_video.payload_specific_options)
   {
      ACU_CHAR *options;
      options = (ACU_CHAR*)malloc(strlen(payloads->payload.audio_video.payload_specific_options)+1);
      strcpy(options, payloads->payload.audio_video.payload_specific_options);
      new_payload->payload.audio_video.payload_specific_options = options;
   }

   if(NULL != payloads->payload.audio_video.rtp_payload_name)
   {
      ACU_CHAR *payload_name;
      payload_name = (ACU_CHAR*)malloc(strlen(payloads->payload.audio_video.rtp_payload_name) + 1);
      strcpy(payload_name, payloads->payload.audio_video.rtp_payload_name);
      new_payload->payload.audio_video.rtp_payload_name = payload_name;
   }
   new_payload->payload.audio_video.rtp_payload_number = payloads->payload.audio_video.rtp_payload_number;

   return new_payload;
}

/*------------------------------------------------------------------------------\
|                                                                               |
|   clone_image_payloads()                                                      |
|                                                                               |
|   Copy payloads for media type ACU_IMAGE                                      |
|                                                                               |
\------------------------------------------------------------------------------*/
ACU_PAYLOAD *clone_image_payloads(const ACU_PAYLOAD *payloads)
{
   ACU_PAYLOAD *new_payload = NULL;
   
   // Although there may be several payloads this function is recursive
   // so only enough memory for one payload needs to be allocated here.
   new_payload = (ACU_PAYLOAD*)malloc(sizeof(ACU_PAYLOAD));
   memset(new_payload, 0, sizeof(ACU_PAYLOAD));

   if(NULL != payloads->next)
   {
      // The next payload in the list is not NULL so it needs to be copied first.
      new_payload->next = clone_image_payloads(payloads->next);
   }

   // Copy payload attributes
   if(NULL != payloads->payload.image.image_payload_name)
   {
      ACU_CHAR* image_payload_name;
      image_payload_name = (ACU_CHAR*)malloc(strlen(payloads->payload.image.image_payload_name) + 1);
      strcpy(image_payload_name, payloads->payload.image.image_payload_name);
      new_payload->payload.image.image_payload_name = image_payload_name;
   }

   return new_payload;
}

/*------------------------------------------------------------------------------\
|                                                                               |
|   clone_control_payloads()                                                    |
|                                                                               |
|   Copy payloads for media type ACU_CONTROL                                    |
|                                                                               |
\------------------------------------------------------------------------------*/
ACU_PAYLOAD *clone_control_payloads(const ACU_PAYLOAD *payloads)
{
   ACU_PAYLOAD *new_payload = NULL;
   
   // Although there may be several payloads this function is recursive
   // so only enough memory for one payload needs to be allocated here.
   new_payload = (ACU_PAYLOAD*)malloc(sizeof(ACU_PAYLOAD));
   memset(new_payload, 0, sizeof(ACU_PAYLOAD));

   if(NULL != payloads->next)
   {
      // The next payload in the list is not NULL so it needs to be copied first.
      new_payload->next = clone_control_payloads(payloads->next);
   }

   // Copy payload attributes
   new_payload->payload.control.dummy = payloads->payload.control.dummy;

   return new_payload;
}
/*------------------------------------------------------------------------------\
|                                                                               |
|   clone_misc_attributes()                                                     |
|                                                                               |
|   Copy ACU_MISCELLANEOUS_MEDIA_ATTRIBUTE linked list                          |
|                                                                               |
\------------------------------------------------------------------------------*/
ACU_MISCELLANEOUS_MEDIA_ATTRIBUTE *clone_misc_attributes(ACU_MISCELLANEOUS_MEDIA_ATTRIBUTE *misc_attributes)
{
   ACU_MISCELLANEOUS_MEDIA_ATTRIBUTE *new_attribute;

   if(NULL == misc_attributes)
   {
      // There are no miscellaneous attributes to copy
      return NULL;
   }

   // Although there may be several attributes this function is recursive
   // so only enough memory for one attribute needs to be allocated here.
   new_attribute = (ACU_MISCELLANEOUS_MEDIA_ATTRIBUTE*)malloc(sizeof(ACU_MISCELLANEOUS_MEDIA_ATTRIBUTE));
   memset(new_attribute, 0, sizeof(ACU_MISCELLANEOUS_MEDIA_ATTRIBUTE));

   if(NULL != misc_attributes->next)
   {
      // The next attribute in the list is not NULL so it needs to be copied first.
      new_attribute->next = clone_misc_attributes(misc_attributes->next);
   }

   // Copy miscellaneous attribute
   if(NULL != misc_attributes->attribute)
   {
      new_attribute->attribute = (ACU_CHAR*)malloc(strlen(misc_attributes->attribute) + 1);
      strcpy(new_attribute->attribute, misc_attributes->attribute);
   }

   return new_attribute;
}

/*------------------------------------------------------------------------------\
|                                                                               |
|   free_media_offer()                                                          |
|                                                                               |
|   Free all dynamically allocated memory in a ACU_MEDIA_OFFER_ANSWER structure |
|                                                                               |
\------------------------------------------------------------------------------*/
void free_media_offer(ACU_MEDIA_OFFER_ANSWER** stored_offer)
{
   if(NULL == *stored_offer)
   {
      return;
   }

   // Free contents of global connection address
   free((*stored_offer)->connection_address.address);
   (*stored_offer)->connection_address.address = NULL;

   // Free the raw SDP char* buffer
   free((*stored_offer)->raw_sdp);
   (*stored_offer)->raw_sdp = NULL;

   // Free any media descriptions
   free_media_descriptions(&((*stored_offer)->media_descriptions));

   // Free the ACU_MEDIA_OFFER_ANSWER structure
   free(*stored_offer);
   *stored_offer = NULL;
}

/*------------------------------------------------------------------------------\
|                                                                               |
|   free_media_descriptions()                                                   |
|                                                                               |
|   Free all dynamically allocated memory in a media description linked list    |
|                                                                               |
\------------------------------------------------------------------------------*/
void free_media_descriptions(ACU_MEDIA_DESCRIPTION **media_descriptions)
{
   if(NULL == *media_descriptions)
   {
      // There are no media descriptions to free
      return;
   }

   if(NULL != (*media_descriptions)->next)
   {
      // The next media description in the list is not NULL so needs to be freed first.
      free_media_descriptions(&((*media_descriptions)->next));
   }

   // Free the local connection address
   free((*media_descriptions)->connection_address.address);
   (*media_descriptions)->connection_address.address = NULL;

   // Free any miscellaneous attributes
   free_misc_attributes(&((*media_descriptions)->miscellaneous_attributes));

   // Free all payloads
   free_payloads(&((*media_descriptions)->payloads), (*media_descriptions)->media_type);

   // Free the transport char* buffer
   free((*media_descriptions)->transport);
   (*media_descriptions)->transport = NULL;

   // Free the ACU_MEDIA_DESCRIPTION structure
   free(*media_descriptions);
   *media_descriptions = NULL;
}

/*------------------------------------------------------------------------------\
|                                                                               |
|   free_payloads()                                                             |
|                                                                               |
|   Check media type and free corresponding payload structures                  |
|                                                                               |
\------------------------------------------------------------------------------*/
void free_payloads(ACU_PAYLOAD **payloads, const ACU_MEDIA_TYPES media_type)
{
   switch(media_type)
   {
   case ACU_AUDIO:
   case ACU_VIDEO:
      free_audio_video_payloads(payloads);
      break;
   case ACU_IMAGE:
      free_image_payloads(payloads);
      break;
   case ACU_CONTROL:
   case ACU_TEXT:
   case ACU_APPLICATION:
      // No dynamically allocated memory here. 
      // Only need to free the pointer to the payload.
      free(*payloads);
      *payloads = NULL;
      break;
   default:
      break;
   }
}

/*------------------------------------------------------------------------------\
|                                                                               |
|   free_audio_video_payloads()                                                 |
|                                                                               |
|   Free payloads for media types ACU_AUDIO and ACU_VIDEO                       |
|                                                                               |
\------------------------------------------------------------------------------*/
void free_audio_video_payloads(ACU_PAYLOAD **payloads)
{
   if(NULL == *payloads)
   {
      // There are no payloads to free
      return;
   }

   if(NULL != (*payloads)->next)
   {
      // The next payload in the list is not NULL so needs to be freed first.
      free_audio_video_payloads(&((*payloads)->next));
   }

   // Free any dynamically allocated elements of the ACU_PAYLOAD.audio_video structure
   free((*payloads)->payload.audio_video.rtp_payload_name);
   (*payloads)->payload.audio_video.rtp_payload_name = NULL;
   free((*payloads)->payload.audio_video.payload_specific_options);
   (*payloads)->payload.audio_video.payload_specific_options = NULL;
   free(*payloads);
   *payloads = NULL;
}

/*------------------------------------------------------------------------------\
|                                                                               |
|   free_image_payloads()                                                       |
|                                                                               |
|   Free payloads for media type ACU_IMAGE                                      |
|                                                                               |
\------------------------------------------------------------------------------*/
void free_image_payloads(ACU_PAYLOAD **payloads)
{
   if(NULL == *payloads)
   {
      // There are no payloads to free
      return;
   }

   if(NULL != (*payloads)->next)
   {
      // The next payload in the list is not NULL so needs to be freed first.
      free_image_payloads(&((*payloads)->next));
   }

   // Free any dynamically allocated elements of the ACU_PAYLOAD.image structure
   free((*payloads)->payload.image.image_payload_name);
   (*payloads)->payload.image.image_payload_name = NULL;
   free(*payloads);
   *payloads = NULL;
}

/*------------------------------------------------------------------------------\
|                                                                               |
|   free_misc_attributes()                                                      |
|                                                                               |
|   Free any ACU_MISCELLANEOUS_MEDIA_ATTRIBUTE structures                       |
|                                                                               |
\------------------------------------------------------------------------------*/
void free_misc_attributes(ACU_MISCELLANEOUS_MEDIA_ATTRIBUTE **misc_attributes)
{
   if(NULL == *misc_attributes)
   {
      // There are no attributes to free
      return;
   }

   if(NULL != (*misc_attributes)->next)
   {
      // The next attribute in the list is not NULL so needs to be freed first.
      free_misc_attributes(&((*misc_attributes)->next));
   }

   // Free other elements of the ACU_MISCELLANEOUS_ATTRIBUTE structure
   free((*misc_attributes)->attribute);
   (*misc_attributes)->attribute = NULL;
   free(*misc_attributes);
   *misc_attributes = NULL;
}
/*------------------------------------------------------------------------------\
|                                                                               |
|   populate_media_answer()                                                     |
|                                                                               |
|   In general the application is free to choose any supported codec contained  |
|   in the media offer. Here, for simplicity, we choose the first supported     |
|   codec found.                                                                |
|                                                                               |
\------------------------------------------------------------------------------*/
ACU_MEDIA_OFFER_ANSWER *populate_media_answer(ACU_MEDIA_OFFER_ANSWER* stored_offer)
{
   ACU_MEDIA_OFFER_ANSWER *answer = (ACU_MEDIA_OFFER_ANSWER*)malloc(sizeof(ACU_MEDIA_OFFER_ANSWER));
   ACU_MEDIA_DESCRIPTION *temp_desc;
   ACU_PAYLOAD *temp_payload, *telephone_event;
   ACU_INT found_supported_codec = 0, found_telephone_event = 0;

   for(temp_desc = stored_offer->media_descriptions; NULL != temp_desc; temp_desc = temp_desc->next)
   {
      for(temp_payload = temp_desc->payloads; NULL != temp_payload; temp_payload = temp_payload->next)
      {
         // Look for first supported codec - essential
         if((0 == found_supported_codec) && (found_supported_codec = codec_is_supported(temp_payload->payload.audio_video.rtp_payload_number)))
         {
            // Found supported codec. Need to copy all details into the media description.
            memset(answer, 0, sizeof(ACU_MEDIA_OFFER_ANSWER));

            answer->media_descriptions = (ACU_MEDIA_DESCRIPTION*)malloc(sizeof(ACU_MEDIA_DESCRIPTION));
            memset(answer->media_descriptions, 0, sizeof(ACU_MEDIA_DESCRIPTION));

            answer->connection_address.address_type     = ACU_IPv4;
            answer->connection_address.address          = (char*)malloc(strlen(LOCAL_CONNECTION_ADDRESS) + 1);
            strcpy(answer->connection_address.address, LOCAL_CONNECTION_ADDRESS);
            answer->media_descriptions->media_direction = ACU_SEND_RECV;
            answer->media_descriptions->media_type      = ACU_AUDIO;
            answer->media_descriptions->packet_length   = temp_desc->packet_length;
            answer->media_descriptions->port            = LOCAL_RTP_PORT;

            answer->media_descriptions->payloads = (ACU_PAYLOAD*)malloc(sizeof(ACU_PAYLOAD));
            memset(answer->media_descriptions->payloads, 0, sizeof(ACU_PAYLOAD));
            answer->media_descriptions->payloads->payload.audio_video.clock_rate         = temp_payload->payload.audio_video.clock_rate;
            answer->media_descriptions->payloads->payload.audio_video.packet_length      = temp_payload->payload.audio_video.packet_length;
            answer->media_descriptions->payloads->payload.audio_video.rtp_payload_number = temp_payload->payload.audio_video.rtp_payload_number;
            if(NULL != temp_payload->payload.audio_video.payload_specific_options)
            {
               char *options;
               options = (char*)malloc(strlen(temp_payload->payload.audio_video.payload_specific_options) + 1);
               strcpy(options, temp_payload->payload.audio_video.payload_specific_options);
               answer->media_descriptions->payloads->payload.audio_video.payload_specific_options = options;
            }
            if(NULL != temp_payload->payload.audio_video.rtp_payload_name)
            {
               char *payload_name;
               payload_name = (char*)malloc(strlen(temp_payload->payload.audio_video.rtp_payload_name) + 1);
               strcpy(payload_name, temp_payload->payload.audio_video.rtp_payload_name);
               answer->media_descriptions->payloads->payload.audio_video.rtp_payload_name = payload_name;
            }
         }

         // Look for telephone event - not essential
         if((0 == found_telephone_event) && (!strcmp(temp_payload->payload.audio_video.rtp_payload_name, ACU_TELEPHONE_EVENT)))
         {
            // Found telephone event. Need to copy all details into the media description.
            found_telephone_event = 1;
            telephone_event = (ACU_PAYLOAD*)malloc(sizeof(ACU_PAYLOAD));
            memset(telephone_event, 0, sizeof(ACU_PAYLOAD));
            telephone_event->payload.audio_video.clock_rate    = temp_payload->payload.audio_video.clock_rate;
            telephone_event->payload.audio_video.packet_length = temp_payload->payload.audio_video.packet_length;
            if(NULL != temp_payload->payload.audio_video.payload_specific_options)
            {
               char *options;
               options = (char*)malloc(strlen(temp_payload->payload.audio_video.payload_specific_options) + 1);
               strcpy(options, temp_payload->payload.audio_video.payload_specific_options);
               telephone_event->payload.audio_video.payload_specific_options = options;
            }
            if(NULL != temp_payload->payload.audio_video.rtp_payload_name)
            {
               char *payload_name;
               payload_name = (char*)malloc(strlen(temp_payload->payload.audio_video.rtp_payload_name) + 1);
               strcpy(payload_name, temp_payload->payload.audio_video.rtp_payload_name);
               telephone_event->payload.audio_video.rtp_payload_name = payload_name;
            }
            telephone_event->payload.audio_video.rtp_payload_number = temp_payload->payload.audio_video.rtp_payload_number;
         }
      }
      if(found_supported_codec)
      {
         if(found_telephone_event)
         {
            answer->media_descriptions->payloads->next = telephone_event;
         }
         break;
      }
   }

   if(!found_supported_codec)
   {
      // If the return value is NULL the application should 
      // reject the call as no supported codec has been found.
      free(answer);
      answer = NULL;
   }

   return answer;
}

/*------------------------------------------------------------------------------\
|                                                                               |
|   populate_media_offer()                                                      |
|                                                                               |
|   For the purposes of these examples any media offer made by the application  |
|   will contain just the one codec and a telpehone-event. The codec used will  |
|   be G.711 as support for this is mandatory for all SIP User Agents.          |
|                                                                               |
\------------------------------------------------------------------------------*/
ACU_MEDIA_OFFER_ANSWER *populate_media_offer(ACU_UINT payload_number, ACU_INT black_hole, ACU_INT no_media)
{
   ACU_MEDIA_OFFER_ANSWER *offer = (ACU_MEDIA_OFFER_ANSWER*)malloc(sizeof(ACU_MEDIA_OFFER_ANSWER));
   ACU_PAYLOAD *codec, *telephone_event;

   // Set the global connection address details. This may be superceded by the local 
   // connection address(es) given in each media description. Here we use the global address.
   memset(offer, 0, sizeof(ACU_MEDIA_OFFER_ANSWER));
   if(black_hole)
   {
      offer->connection_address.address = (ACU_CHAR*)malloc(strlen(BLACKHOLE_CONNECTION_ADDRESS) + 1);
      strcpy(offer->connection_address.address, BLACKHOLE_CONNECTION_ADDRESS);
   }
   else
   {
      offer->connection_address.address = (ACU_CHAR*)malloc(sizeof(LOCAL_CONNECTION_ADDRESS) + 1);
      strcpy(offer->connection_address.address, LOCAL_CONNECTION_ADDRESS);
   }
   offer->connection_address.address_type = ACU_IPv4;

   if(0 == no_media)
   {
      // Set just one media description. Again, there may be more.
      offer->media_descriptions = (ACU_MEDIA_DESCRIPTION*)malloc(sizeof(ACU_MEDIA_DESCRIPTION));
      memset(offer->media_descriptions, 0, sizeof(ACU_MEDIA_DESCRIPTION));
      offer->media_descriptions->media_direction   = ACU_SEND_RECV;
      offer->media_descriptions->media_type        = ACU_AUDIO;
      offer->media_descriptions->port              = local_rtp_port;

      // Usually an application will present a choice of codecs to the far end.
      // Here we use G.711 as support for it is mandatory.
      codec = (ACU_PAYLOAD*)malloc(sizeof(ACU_PAYLOAD));
      memset(codec, 0, sizeof(ACU_PAYLOAD));
      codec->payload.audio_video.clock_rate           = ACU_EIGHT_K;
      codec->payload.audio_video.rtp_payload_number   = payload_number;
      codec->payload.audio_video.rtp_payload_name     = convert_payload_number_to_name(payload_number);
      if(NULL == codec->payload.audio_video.rtp_payload_name)
      {
         free(offer);
         return NULL;
      }

      // Add a telephone event.
      telephone_event = (ACU_PAYLOAD*)malloc(sizeof(ACU_PAYLOAD));
      memset(telephone_event, 0, sizeof(ACU_PAYLOAD));
      telephone_event->payload.audio_video.clock_rate                = ACU_EIGHT_K;
      telephone_event->payload.audio_video.rtp_payload_number        = 101;
      telephone_event->payload.audio_video.payload_specific_options  = (char*)malloc(strlen("0-15") + 1);
      strcpy(telephone_event->payload.audio_video.payload_specific_options, "0-15");
      telephone_event->payload.audio_video.rtp_payload_name          = (char*)malloc(strlen(ACU_TELEPHONE_EVENT) + 1);
      strcpy(telephone_event->payload.audio_video.rtp_payload_name, ACU_TELEPHONE_EVENT);

      codec->next = telephone_event;
      offer->media_descriptions->payloads = codec;
   }

   return offer;
}

/*------------------------------------------------------------------------------\
|                                                                               |
|   codec_is_supported()                                                        |
|                                                                               |
|   Return TRUE if the supplied payload type is supported by Aculab cards.      |
|   Return FALSE otherwise.                                                     |
|                                                                               |
\------------------------------------------------------------------------------*/
ACU_INT codec_is_supported(const ACU_INT payload_number)
{
   ACU_INT result = 0;

   switch(payload_number)
   {
   case ACU_PCMU_PAYLOAD_NUMBER:  // G.711 mu-law
   case ACU_G723_PAYLOAD_NUMBER:  // G.723
   // case ACU_PCMA_PAYLOAD_NUMBER:  // G.711 a-law
   case ACU_G729_PAYLOAD_NUMBER:  // G.729
      result = 1;
      break;
   }

   return result;
}

/*------------------------------------------------------------------------------\
|                                                                               |
|   convert_payload_number_to_name()                                            |
|                                                                               |
|   Takes a payload number (e.g. ACU_G723_PAYLOAD_NUMBER) and return a payload  |
|   name (e.g. ACU_G723).                                                       |
|                                                                               |
\------------------------------------------------------------------------------*/
ACU_CHAR *convert_payload_number_to_name(const ACU_UINT payload_number)
{
   ACU_CHAR* payload_name = (ACU_CHAR*)malloc(32);

   switch(payload_number)
   {
   case ACU_PCMU_PAYLOAD_NUMBER:  // G.711 mu-law
      strcpy(payload_name, ACU_PCMU);
      break;
   case ACU_G723_PAYLOAD_NUMBER:  // G.723
      strcpy(payload_name, ACU_G723);
      break;
   case ACU_PCMA_PAYLOAD_NUMBER:  // G.711 a-law
      strcpy(payload_name, ACU_PCMA);
      break;
   case ACU_G729_PAYLOAD_NUMBER:  // G.729
      strcpy(payload_name, ACU_G729);
      break;
   default:
      free(payload_name);
      payload_name = NULL;
      break;
   }

   return payload_name;
}
