//*******************************************************************
//
// Company:		Sangoma Technologies, Inc.
//
// Author:		David Rokhvarg <davidr@sangoma.com>
//
// FileName:	sangoma_interface_manager.cpp
//
// Description:	keeps a list of available Sangoma API interfaces
//
// Version:		1.0		05/7/2007	initial version
//
//******************************************************************

#include "sangoma_interface_manager.h"

//#define BRKP() { printf("line: %d\n", __LINE__);	DebugBreak(); }
#define BRKP()

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

sangoma_interface_manager::sangoma_interface_manager()
{
	const char *FUNC_NAME = "sangoma_interface_manager";

	_snprintf(CLASS_NAME, 100, "sangoma_interface_manager");

	IF_MANAGER_FUNC();

	InitializeCriticalSection(&IfMangerCriticalSection);
}

sangoma_interface_manager::~sangoma_interface_manager()
{
	const char *FUNC_NAME = "~sangoma_interface_manager";

	IF_MANAGER_FUNC();

	flush_list_of_interfaces();

	DeleteCriticalSection(&IfMangerCriticalSection);
}

int sangoma_interface_manager::start()
{
	const char *FUNC_NAME = "start";
	IF_MANAGER_FUNC();
	unsigned int	rc = 0;

	EnterCriticalSection(&IfMangerCriticalSection); 

	list<sangoma_interface *>::iterator if_iterator;

	DBG_SANG_IF_MANAGER("if_list.size(): %d\n", if_list.size());

	for(if_iterator = if_list.begin(); if_iterator != if_list.end(); ++if_iterator){

		((sangoma_interface *)*if_iterator)->enable_events();

		rc = ((sangoma_interface *)*if_iterator)->run();
		if(rc){
			//error
			break;
		}
	}

	LeaveCriticalSection(&IfMangerCriticalSection);

	return rc;
}

int sangoma_interface_manager::build_list_of_interfaces(sangoma_callback_functions_t *cb)
{
	const char *FUNC_NAME = "build_list_of_interfaces";
	int	  init_status = 0;

	IF_MANAGER_FUNC();

	EnterCriticalSection(&IfMangerCriticalSection); 

	for(int card = 0; card < MAX_NUM_OF_CARDS; card++){
		for(int line = 0; line < MAX_NUM_OF_LINES; line++){

			sangoma_interface *sang_if = new sangoma_interface(card, line);
			if(sang_if->init(cb)){
				delete sang_if;
				continue;
			}

			if(sang_if->wanif_conf_struct.media == WAN_MEDIA_FXOFXS){
				switch(sang_if->wanif_conf_struct.sub_media)
				{
				case MOD_TYPE_FXS:
				case MOD_TYPE_FXO:
					if_list.push_back(sang_if);// Insert a new element at the end of the list
					break;
				default:
					ERR_SANG_IF_MANAGER("ERROR: %s():Invalid Media sub-type\t\t: Unknown!!\n",
						FUNC_NAME);
					LeaveCriticalSection(&IfMangerCriticalSection);
					return 1;
				}//switch()
			}else{
				//not an Analog card (could be Sangoma T1/E1). not used by this program.
				delete sang_if;
			}//if()
		}//for()
	}//for()

	if(if_list.size() == 0){
		ERR_SANG_IF_MANAGER("ERROR: %s(): No Sangoma cards detected!\n",
				FUNC_NAME);
		LeaveCriticalSection(&IfMangerCriticalSection);
		return 1;
	}

	LeaveCriticalSection(&IfMangerCriticalSection);

	//start all interfaces in the list
	return start();
}

int sangoma_interface_manager::flush_list_of_interfaces()
{
	const char *FUNC_NAME = "flush_list_of_interfaces";
	IF_MANAGER_FUNC();

	EnterCriticalSection(&IfMangerCriticalSection); 

	list<sangoma_interface *>::iterator if_iterator;

	DBG_SANG_IF_MANAGER("if_list.size(): %d\n", if_list.size());

	//1. stop all interfaces in the list
	//2. delete ALL objects referenced by pointers in the list
	for(if_iterator = if_list.begin(); if_iterator != if_list.end(); ++if_iterator){

		DBG_SANG_IF_MANAGER("(3) Stopping %s\n", ((sangoma_interface *)*if_iterator)->device_name);

		delete (sangoma_interface *)*if_iterator;
	}//for()

	//remove all the pointers from the list
	while(if_list.size()){
		if_list.pop_front();
	}

	LeaveCriticalSection(&IfMangerCriticalSection);
	return 0;
}

sangoma_interface *sangoma_interface_manager::find_free_interface()
{
	const char *FUNC_NAME = "find_free_interface";
	DBG_SANG_IF_MANAGER("%s()\n", FUNC_NAME);

	list<sangoma_interface *>::iterator if_iterator;

	DBG_SANG_IF_MANAGER("if_list.size(): %d\n", if_list.size());

	//delete ALL objects referenced by pointers in the list
	for(if_iterator = if_list.begin(); if_iterator != if_list.end(); ++if_iterator){
		if(((sangoma_interface *)*if_iterator)->get_if_state("find_free_interface()") == IF_FREE){

			DBG_SANG_IF_MANAGER("Found FREE interface: %s\n",
				((sangoma_interface *)*if_iterator)->device_name);
			return (sangoma_interface *)*if_iterator;
		}
	}

	DBG_SANG_IF_MANAGER("ALL interfaces are BUSY!\n");

	return NULL;
}



